#!/usr/bin/perl -w

#######################################################################
# Modules
#######################################################################
use strict;
use Getopt::Std;
use File::Basename;
use Math::BigInt lib => 'Calc';


#######################################################################
# Global variables
#######################################################################
my %opts = ();
my $self = basename($0);
my $ssid = -1;
my $pool = "(undef)";
my %fragments = ();
my $verbose = 0;


#######################################################################
# Functions
#######################################################################

sub usage {
	print <<EOF;
$self [-h] -S ssid [-b pool] [-v]

Where:

	-h	Print this help and exit.
	-S ssid	Print information for saveset denoted by ssid.
	-b pool	Restrict search to nominated pool.
	-v	Enable verbose mode.

For use with savesets that span multiple tapes, this command
prints out the order in which savesets appear on the tape(s)
for correct rescanning.
EOF

	if (@_+0 != 0) {
		my @messages = @_;
		foreach my $line (@messages) {
			my $tmp = $line;
			chomp $tmp;
			print "$tmp\n";
		}
	}
	die "\n";
}


#######################################################################
# Main
#######################################################################
if (getopts('hS:b:',\%opts)) {
	usage() if (defined($opts{h}));

	if (defined($opts{S}) && $opts{S} =~ /^\d+$/) {
		$ssid = $opts{S};
	} else {
		usage("Saveset ID must be positive integer.\n");
	}

	if (defined($opts{v})) {
		$verbose = 1;
	}

	if (defined($opts{b})) {
		$pool = $opts{b};
	}
}

# Assemble mminfo query.
my $query = "mminfo -q \"ssid=$ssid";
if ($pool ne "(undef)") {
	$query .= ",pool=$pool\"";
} else {
	$query .= "\"";
}

$query .= " -r volume,first,last -xc,";
($verbose) && print "Query: $query\n\n";

if (open(MMI,"$query 2>&1 |")) {
	<MMI>;
	while (<MMI>) {
		my $tmp = $_;
		chomp $tmp;
		my @entries = split(/,/,$tmp);
		#my ($volume,$first,$last) = (split(/,/,$tmp))[0,1,2];
		my $volume = $entries[0];
		my $first = Math::BigInt->new($entries[1]);
		my $last = Math::BigInt->new($entries[2]);
	
		$fragments{$first}{volume} = $volume;
		$fragments{$first}{end} = $last;

	}
	close(MMI);
} else {
	die "Could not execute: $query\n";
}

my @volumes = ();
my $volCount = 0;
foreach my $fragment (sort {$a <=> $b} keys %fragments) {
	($verbose) && print "$fragment: $fragments{$fragment}{volume} -> $fragments{$fragment}{end}\n";
	push(@volumes,$fragments{$fragment}{volume});
	$volCount++;
}

my $volString = "$volCount volume";
$volString .= "s" if ($volCount > 1);

print "\n$volString to be scanned in the following order:\n" . join(", ",@volumes) . "\n";
